{ Demonstration program for ChroMathsome - binary logic.
  
  Author: John Merrit (john.merrit@net4all.be)
  Release date: 25 May '97

  Program is a fully working multi-level stack RPN calculator to perform
  binary operations.
  Calculator can have 100's of arguments on its stack.
  Design is based on the Hewlett Packard 48GX Advanced Scientific RPN calculator.


  P.S.
  This program was intentionally made for the shareware market.
  Since it shows some nice techniques (how to write a HP like RPN calculator,
  writing a keyboard interface, etc...) and yours truly is a (really) nice guy (;-))
  -> you may have it.
  I wouldn't mind a little credit if you use the code below as part of or in
  a commercial or shareware program.
  After all, I wrote this program.

  Read the BinLogic help file for more information about wordsizes and the implemented
  functions.


  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "R".
                }


unit BinCal;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, BinLogic;

type
  TForm1 = class(TForm)
    EButton: TButton;
    XORButton: TButton;
    DButton: TButton;
    BButton: TButton;
    DivButton: TButton;
    FButton: TButton;
    OrButton: TButton;
    CButton: TButton;
    EightButton: TButton;
    AButton: TButton;
    RLButton: TButton;
    NineButton: TButton;
    SixButton: TButton;
    SevenButton: TButton;
    RLBButton: TButton;
    MultiplyButton: TButton;
    FiveButton: TButton;
    FourButton: TButton;
    MinusButton: TButton;
    ThreeButton: TButton;
    TwoButton: TButton;
    OneButton: TButton;
    PlusButton: TButton;
    AboutButton: TButton;
    EnterButton: TButton;
    NilButton: TButton;
    ASRButton: TButton;
    SLBButton: TButton;
    RRButton: TButton;
    SRButton: TButton;
    SLButton: TButton;
    NotButton: TButton;
    AndButton: TButton;
    SetWordSizeButton: TButton;
    DropButton: TButton;
    RecallWordSizeButton: TButton;
    DuplicateButton: TButton;
    SRBButton: TButton;
    RRBButton: TButton;
    OctButton: TButton;
    BinButton: TButton;
    DeleteButton: TButton;
    ClearButton: TButton;
    SwapButton: TButton;
    DecButton: TButton;
    HexButton: TButton;
    RPNScreen: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    InputEdit: TEdit;
    NEGButton: TButton;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EnterButtonClick(Sender: TObject);
    procedure DropButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure NumBut(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure BaseButton(Sender: TObject);
    procedure AdjustWordSize(Sender: TObject);
    procedure DuplicateButtonClick(Sender: TObject);
    procedure Arithmetic(Sender: TObject);
    procedure OneArgument(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure InputEditChange(Sender: TObject);
    procedure InputEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SwapButtonClick(Sender: TObject);
    procedure AboutButtonClick(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateScreen;
    procedure ShowError(ErrorType: integer);
    procedure ConvertNumbers(OldBase, NewBase: NumberBase);
    procedure UpdateButtons(NewBase: NumberBase);
    procedure FocusInput;

  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  RPNList: TStringList;
  Op1, Op2, Op3: string;
  OldBase, NewBase: NumberBase;
  OldSize, NewSize: integer;
  ErrorOccured: boolean;

implementation

{$R *.DFM}


procedure TForm1.Exit1Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
     Op1:= '';
     Op2:= '';
     Op3:= '';
     OldBase:= Hex;
     OldSize:= 64;
     ErrorOccured:= false;
     Form1.Caption:= 'Binary RPN Calculator';
     RPNList:= TStringList.Create;
     RPNScreen.Lines[0]:= '';
     RPNScreen.Lines[1]:= '';
     RPNScreen.Lines[2]:= '';
     RPNScreen.Lines[3]:= '';
     RPNScreen.Lines[4]:= '';
end;

{ Define errors.
  Note that ChroMathsome - Binary Logic has its own exception handling mechanism.
  If, for example, a division by zero error occurs, ChroMathsome - Binary Logic will
  pop-up a message box.
  Some of the errors below are only implemented to show how you can add an error
  mechanism to a calculator.                                                      }
procedure TForm1.ShowError(ErrorType: integer);
begin
     EnterButton.Enabled:= false;
     case ErrorType of
     1:RpnScreen.Lines[0]:= 'Error: Too few arguments';
     2:RpnScreen.Lines[0]:= 'Error: Invalid argument(s)';
     3:RpnScreen.Lines[0]:= 'Error: Overflow';
     4:RpnScreen.Lines[0]:= 'Error: Division by zero';
     5:RpnScreen.Lines[0]:= 'Error: Invalid operation';
     end;
     ErrorOccured:= true;
     RpnScreen.Lines[1]:= 'Press Clear [CLR]';
end;

procedure TForm1.FocusInput;
begin
     InputEdit.SetFocus;
     InputEdit.Selstart:= length(InputEdit.text);
     InputEdit.Sellength:= 0;
end;

procedure TForm1.UpdateScreen;
begin
     if RPNList.Count = 0 then
     begin
     RPNScreen.Lines[0]:= '';
     RPNScreen.Lines[1]:= '';
     RPNScreen.Lines[2]:= '';
     RPNScreen.Lines[3]:= '';
     RPNScreen.Lines[4]:= '';
     end
     else if RPNList.Count-1 = 0 then
     begin
     RPNScreen.Lines[0]:= '';
     RPNScreen.Lines[1]:= '';
     RPNScreen.Lines[2]:= '';
     RPNScreen.Lines[3]:= '';
     RPNScreen.Lines[4]:= RPNList.Strings[RPNList.Count-1];
     end
     else if RPNList.Count-1 = 1 then
     begin
     RPNScreen.Lines[0]:= '';
     RPNScreen.Lines[1]:= '';
     RPNScreen.Lines[2]:= '';
     RPNScreen.Lines[3]:= RPNList.Strings[RPNList.Count-2];
     RPNScreen.Lines[4]:= RPNList.Strings[RPNList.Count-1];
     end
     else if RPNList.Count-1 = 2 then
     begin
     RPNScreen.Lines[0]:= '';
     RPNScreen.Lines[1]:= '';
     RPNScreen.Lines[2]:= RPNList.Strings[RPNList.Count-3];
     RPNScreen.Lines[3]:= RPNList.Strings[RPNList.Count-2];
     RPNScreen.Lines[4]:= RPNList.Strings[RPNList.Count-1];
     end
     else if RPNList.Count-1 = 3 then
     begin
     RPNScreen.Lines[0]:= '';
     RPNScreen.Lines[1]:= RPNList.Strings[RPNList.Count-4];
     RPNScreen.Lines[2]:= RPNList.Strings[RPNList.Count-3];
     RPNScreen.Lines[3]:= RPNList.Strings[RPNList.Count-2];
     RPNScreen.Lines[4]:= RPNList.Strings[RPNList.Count-1];
     end
     else
     begin
     RPNScreen.Lines[0]:= RPNList.Strings[RPNList.Count-5];
     RPNScreen.Lines[1]:= RPNList.Strings[RPNList.Count-4];
     RPNScreen.Lines[2]:= RPNList.Strings[RPNList.Count-3];
     RPNScreen.Lines[3]:= RPNList.Strings[RPNList.Count-2];
     RPNScreen.Lines[4]:= RPNList.Strings[RPNList.Count-1];
     end;
    
     Op1:= '';
     Op2:= '';
     FocusInput;
end;

procedure TForm1.EnterButtonClick(Sender: TObject);
begin
     if InputEdit.Text = '' then ShowError(1)
     else
     begin
     InputEdit.Text:= DeleteLeadingZeroes(InputEdit.Text);
     RPNList.Add(UpperCase(InputEdit.Text));
     UpdateScreen;
     InputEdit.Text:= '';
     Op1:= '';
     Op2:= '';
     Op3:= '';
     end;
     FocusInput;
end;

procedure TForm1.DropButtonClick(Sender: TObject);
begin
     if ((InputEdit.Text = '') and (RpnScreen.Lines[4] = '')) then ShowError(1)
     else
     begin
     if InputEdit.Text = '' then
     begin
     RPNList.Delete(RPNList.Count-1);
     UpdateScreen;
     end
     else
     InputEdit.Text:= '';
     Op1:= '';   { Don't forget this one! }
     end;
     FocusInput;
end;

procedure TForm1.DuplicateButtonClick(Sender: TObject);
begin
     if ((InputEdit.Text = '') and (RpnScreen.Lines[4] = '')) then ShowError(1)
     else
     begin
     if InputEdit.Text = '' then
     begin
     InputEdit.Text:= RpnScreen.Lines[4];
     EnterButtonClick(Sender);
     end
     else
     begin
     EnterButtonClick(Sender);
     InputEdit.Text:= RpnScreen.Lines[4];
     end;
     end;
     FocusInput;
end;

procedure TForm1.DeleteButtonClick(Sender: TObject);
begin
     InputEdit.Text:= '';
     InputEdit.SetFocus;
end;

{ Handle number buttons. }
procedure TForm1.NumBut(Sender: TObject);
begin
     if Sender = NilButton then Op1:= Op1 + '0'
     else if Sender = OneButton then Op1:= Op1 + '1'
     else if Sender = TwoButton then Op1:= Op1 + '2'
     else if Sender = ThreeButton then Op1:= Op1 + '3'
     else if Sender = FourButton then Op1:= Op1 + '4'
     else if Sender = FiveButton then Op1:= Op1 + '5'
     else if Sender = SixButton then Op1:= Op1 + '6'
     else if Sender = SevenButton then Op1:= Op1 + '7'
     else if Sender = EightButton then Op1:= Op1 + '8'
     else if Sender = NineButton then Op1:= Op1 + '9'
     else if Sender = AButton then Op1:= Op1 + 'A'
     else if Sender = BButton then Op1:= Op1 + 'B'
     else if Sender = CButton then Op1:= Op1 + 'C'
     else if Sender = DButton then Op1:= Op1 + 'D'
     else if Sender = EButton then Op1:= Op1 + 'E'
     else if Sender = FButton then Op1:= Op1 + 'F';
     InputEdit.Text:= Op1;
     FocusInput;
end;

{ Clear the stack or error if an error has occured. }
procedure TForm1.ClearButtonClick(Sender: TObject);
begin
     if ErrorOccured = false then
     begin
     Op1:= '';
     Op2:= '';
     Op3:= '';
     RPNList.Clear;
     UpDateScreen;
     InputEdit.Text:= '';
     end
     else
     begin
     ErrorOccured:= false;
     UpdateScreen;
     EnterButton.Enabled:= true;
     end;
     FocusInput;
end;

procedure TForm1.BaseButton(Sender: TObject);
begin
     if Sender = HexButton then NewBase:= Hex
     else if Sender = DecButton then NewBase:= Dec
     else if Sender = BinButton then NewBase:= Bin
     else NewBase:= Oct;
     if NewBase <> OldBase then
     begin
     ConvertNumbers(OldBase, NewBase);
     OldBase:= NewBase;
     UpdateButtons(NewBase);
     end;
     FocusInput;
end;

{ Update calculator screen.
  Just like with the HP48, all stack levels will be updated. }
procedure TForm1.ConvertNumbers(OldBase, NewBase: NumberBase);
var s1: string;
    i: integer;
begin
     s1:= InputEdit.Text;
     case OldBase of
     Dec:begin
         case NewBase of
         Bin:begin
             if s1 <> '' then InputEdit.Text:= DecToBin(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= DecToBin(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Oct:begin
             if s1 <> '' then InputEdit.Text:= DecToOct(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= DecToOct(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Hex:begin
             if s1 <> '' then InputEdit.Text:= DecToHex(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= DecToHex(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         end; // case
         end;
     Bin:begin
         case NewBase of
         Dec:begin
             if s1 <> '' then InputEdit.Text:= BinToDec(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= BinToDec(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Oct:begin
             if s1 <> '' then InputEdit.Text:= BinToOct(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= BinToOct(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Hex:begin
             if s1 <> '' then InputEdit.Text:= BinToHex(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= BinToHex(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         end; // case
         end;
     Oct:begin
         case NewBase of
         Bin:begin
             if s1 <> '' then InputEdit.Text:= OctToBin(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= OctToBin(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Dec:begin
             if s1 <> '' then InputEdit.Text:= OctToDec(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= OctToDec(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Hex:begin
             if s1 <> '' then InputEdit.Text:= OctToHex(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= OctToHex(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         end; // case
         end;
     Hex:begin
         case NewBase of
         Bin:begin
             if s1 <> '' then InputEdit.Text:= HexToBin(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= HexToBin(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Oct:begin
             if s1 <> '' then InputEdit.Text:= HexToOct(s1, OldSize);
             for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= HexToOct(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         Dec:begin
             if s1 <> '' then InputEdit.Text:= HexToDec(s1, OldSize);
            for i:= 0 to RPNList.Count-1 do
                  begin
                  s1:= RPNList.Strings[i];
                  if s1 <> '' then s1:= HexToDec(s1, OldSize);
                  RPNList.Strings[i]:= s1;
                  end;
             end;
         end; // case
         end;
     end; // case
     UpDateScreen;
end;

{ When user changes binary base, update buttons... }
procedure TForm1.UpdateButtons(NewBase: NumberBase);
begin
     case NewBase of
     Dec:begin
         TwoButton.Enabled:= true;
         ThreeButton.Enabled:= true;
         FourButton.Enabled:= true;
         FiveButton.Enabled:= true;
         SixButton.Enabled:= true;
         SevenButton.Enabled:= true;
         EightButton.Enabled:= true;
         NineButton.Enabled:= true;
         AButton.Enabled:= false;
         BButton.Enabled:= false;
         CButton.Enabled:= false;
         DButton.Enabled:= false;
         EButton.Enabled:= false;
         FButton.Enabled:= false;
         DecButton.Caption:= '<DEC>';
         BinButton.Caption:= 'BIN';
         OctButton.Caption:= 'OCT';
         HexButton.Caption:= 'HEX';
         end;
     Bin:begin
         TwoButton.Enabled:= false;
         ThreeButton.Enabled:= false;
         FourButton.Enabled:= false;
         FiveButton.Enabled:= false;
         SixButton.Enabled:= false;
         SevenButton.Enabled:= false;
         EightButton.Enabled:= false;
         NineButton.Enabled:= false;
         AButton.Enabled:= false;
         BButton.Enabled:= false;
         CButton.Enabled:= false;
         DButton.Enabled:= false;
         EButton.Enabled:= false;
         FButton.Enabled:= false;
         DecButton.Caption:= 'DEC';
         BinButton.Caption:= '<BIN>';
         OctButton.Caption:= 'OCT';
         HexButton.Caption:= 'HEX';
         end;
     Oct:begin
         TwoButton.Enabled:= true;
         ThreeButton.Enabled:= true;
         FourButton.Enabled:= true;
         FiveButton.Enabled:= true;
         SixButton.Enabled:= true;
         SevenButton.Enabled:= true;
         EightButton.Enabled:= false;
         NineButton.Enabled:= false;
         AButton.Enabled:= false;
         BButton.Enabled:= false;
         CButton.Enabled:= false;
         DButton.Enabled:= false;
         EButton.Enabled:= false;
         FButton.Enabled:= false;
         DecButton.Caption:= 'DEC';
         BinButton.Caption:= 'BIN';
         OctButton.Caption:= '<OCT>';
         HexButton.Caption:= 'HEX';
         end;
     Hex:begin
         TwoButton.Enabled:= true;
         ThreeButton.Enabled:= true;
         FourButton.Enabled:= true;
         FiveButton.Enabled:= true;
         SixButton.Enabled:= true;
         SevenButton.Enabled:= true;
         EightButton.Enabled:= true;
         NineButton.Enabled:= true;
         AButton.Enabled:= true;
         BButton.Enabled:= true;
         CButton.Enabled:= true;
         DButton.Enabled:= true;
         EButton.Enabled:= true;
         FButton.Enabled:= true;
         DecButton.Caption:= 'DEC';
         BinButton.Caption:= 'BIN';
         OctButton.Caption:= 'OCT';
         HexButton.Caption:= '<HEX>';
         end;
     end; // case;
end;

{ User can only adjust/and see current wordsize if base is decimal (Dec). }
procedure TForm1.AdjustWordSize(Sender: TObject);
begin
     if OldBase <> Dec then ShowError(5)
     else
     begin
     if Sender = SetWordSizeButton then
     begin
     if ((InputEdit.Text = '') and (RpnScreen.Lines[4] = '')) then ShowError(1)
     else
     begin
     if InputEdit.Text = '' then
     NewSize:= StrToInt(RpnScreen.Lines[4])
     else
     NewSize:= StrToInt(InputEdit.Text);
     if NewSize <> OldSize then
     begin
     ConvertNumbers(OldBase, NewBase);
     OldSize:= NewSize;
     end;
     DropButtonClick(Sender);
     end;
     end
     else // Show current wordsize
     ShowMessage('Current wordsize = ' + IntToStr(OldSize));
     end;
     FocusInput;
end;

{ Binary arithmetic. }
procedure TForm1.Arithmetic(Sender: TObject);
begin
     Op3:= '';
     if RpnScreen.Lines[4] = '' then ShowError(1)
     else if ((InputEdit.Text = '') and (RpnScreen.Lines[3] = '')) then ShowError(1)
     else
     begin
     if InputEdit.Text = '' then
     begin
     Op1:= RpnScreen.Lines[3];
     Op2:= RpnScreen.Lines[4];
     end
     else
     begin
     Op1:= RpnScreen.Lines[4];
     Op2:= InputEdit.Text;
     end;
     if Sender = PlusButton then Op3:= BinAdd(Op1, Op2, OldBase, OldSize)
     else if Sender = MinusButton then Op3:= BinSubtract(Op1, Op2, OldBase, OldSize)
     else if Sender = MultiplyButton then Op3:= BinMultiply(Op1, Op2, OldBase, OldSize)
     else if Sender = DivButton then Op3:= BinDivide(Op1, Op2, OldBase, OldSize)
     else if Sender = OrButton then Op3:= BinOR(Op1, Op2, OldBase, OldSize)
     else if Sender = XOrButton then Op3:= BinXOR(Op1, Op2, OldBase, OldSize)
     else if Sender = AndButton then Op3:= BinAND(Op1, Op2, OldBase, OldSize);
     DropButtonClick(Sender);
     DropButtonClick(Sender);  { Yes, 2 times }
     InputEdit.Text:= Op3;
     EnterButtonClick(Sender);
     end;
end;

{ Functions that require only one argument. }
procedure TForm1.OneArgument(Sender: TObject);
begin
     Op3:= '';
     if ((InputEdit.Text = '') and (RpnScreen.Lines[4] = '')) then ShowError(1)
     else
     begin
     if InputEdit.Text = '' then
     Op1:= RpnScreen.Lines[4]
     else
     Op1:= InputEdit.Text;
     if Sender = NegButton then Op3:= BinNegative(Op1, OldBase, OldSize)
     else if Sender = NotButton then Op3:= BinNegative(Op1, OldBase, OldSize)
     else if Sender = RLButton then Op3:= BinRotateLeftBit(Op1, OldBase, OldSize)
     else if Sender = SLButton then Op3:= BinShiftLeftBit(Op1, OldBase, OldSize)
     else if Sender = SRButton then Op3:= BinShiftRightBit(Op1, OldBase, OldSize)
     else if Sender = RRButton then Op3:= BinRotateRightBit(Op1, OldBase, OldSize)
     else if Sender = RLBButton then Op3:= BinRotateLeftByte(Op1, OldBase, OldSize)
     else if Sender = SLBButton then Op3:= BinShiftLeftByte(Op1, OldBase, OldSize)
     else if Sender = SRBButton then Op3:= BinShiftRightByte(Op1, OldBase, OldSize)
     else if Sender = RRBButton then Op3:= BinRotateRightByte(Op1, OldBase, OldSize)
     else if Sender = ASRButton then Op3:= ArithmeticShiftRightBit(Op1, OldBase, OldSize);
     DropButtonClick(Sender);
     InputEdit.Text:= Op3;
     EnterButtonClick(Sender);
     end;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
     InputEdit.SetFocus;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     RPNList.Free;
end;

{ Make sure the user can only enter valid numbers in the specified base.
  If user types invalid character, input character will be ignored. }
procedure TForm1.InputEditChange(Sender: TObject);
var s: string;
begin
     if InputEdit.Text <> '' then
     begin
     if IsValidNumber(InputEdit.Text, OldBase) = false then
     begin
     if Length(InputEdit.Text) = 1 then InputEdit.Text:= ''
     else
     begin
     s:= InputEdit.Text;
     Delete(s, Length(s), 1);
     InputEdit.Text:= s;
     InputEdit.Selstart:= length(InputEdit.text);
     InputEdit.Sellength:= 0;
     end;
     end;
     end;
end;

{ Keyboard interface for Enter, Clear, Delete, +, -, *, / buttons.
  Note that for number buttons (0, .., 9, A, .., F) we do not need to
  'trap' the keys.
  Look at Virtual KeyCodes in the Delphi help file for key codes.
  Other buttons can also be assigned a keyboard button, do this yourself. }
procedure TForm1.InputEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
      case key of
      $0D:if ErrorOccured = false then EnterButtonClick(Sender);
      $11:ClearButtonClick(Sender);
      $2E:DeleteButtonClick(Sender);
      $6B, $6D, $6A, $6F:
          begin
          case Key of
          $6B:Sender:= PlusButton;
          $6D:Sender:= MinusButton;
          $6A:Sender:= MultiplyButton;
          $6F:Sender:= DivButton;
          end;
          Arithmetic(Sender);
          end;
      end; // case
end;

procedure TForm1.SwapButtonClick(Sender: TObject);
var s: string;
begin
     if InputEdit.Text = '' then
     begin
     if RPNList.Count < 2 then ShowError(1)
     else
     begin
     RpnList.Exchange(RPNList.Count-1, RPNList.Count-2);
     UpdateScreen;
     end;
     end
     else
     begin
     if ((InputEdit.Text = '') or (RpnList.Count = 0)) then ShowError(1)
     else
     begin
     s:= RpnList.Strings[RPNList.Count-1];
     RPNList.Delete(RPNList.Count-1);
     RpnList.Add(InputEdit.Text);
     InputEdit.Text:= s;
     InputEdit.Selstart:= length(InputEdit.text);
     InputEdit.Sellength:= 0;
     UpdateScreen;
     end;
     end;
end;

procedure TForm1.AboutButtonClick(Sender: TObject);
begin
     MessageDlg('Binary Logic multi-level stack rpn calculator.' + #13 +
     'Copyright 1997 John Merrit.' + #13 + 'All Rights Reserved.' +
      #13 + 'Brought to you by the letters "M" and "R".', mtInformation,[mbOk], 0);
     FocusInput;
end;

end.
